/*
 * Board functions for Atmark Techno Armadillo-IoT Subboard
 *
 * Copyright (C) 2021 Atmark Techno, Inc.
 *
 * Mitsuhiro Yoshida <mitsuhiro.yoshida@atmark-techno.com>
 *
 * SPDX-License-Identifier:	GPL-2.0+
 *
 */

#include <asm/types.h>
#include <asm/arch/iomux.h>
#include <asm/arch/mx6-pins.h>
#include <asm/gpio.h>
#include <common.h>
#include <i2c.h>
#include "a6_subboard.h"
#include "board_i2c.h"

/*
#define GPIO_I2C_PAD_CTRL	(PAD_CTL_PUS_100K_UP | PAD_CTL_SPEED_LOW | \
	PAD_CTL_DSE_120ohm | PAD_CTL_HYS | PAD_CTL_PUE | PAD_CTL_PKE )
 */
#define GPIO_I2C_PAD_CTRL	(PAD_CTL_PUS_100K_UP | PAD_CTL_SPEED_LOW | \
	PAD_CTL_DSE_120ohm | PAD_CTL_HYS | PAD_CTL_PUE | PAD_CTL_PKE )

struct i2c_pads_info i2c_gpio_pad = {
	.scl = {
		.gpio_mode = MX6_PAD_UART5_TX_DATA__GPIO1_IO30 | MUX_PAD_CTRL(GPIO_I2C_PAD_CTRL),
		.gp = IMX_GPIO_NR(1, 30),
	},
	.sda = {
		.gpio_mode = MX6_PAD_UART5_RX_DATA__GPIO1_IO31 | MUX_PAD_CTRL(GPIO_I2C_PAD_CTRL),
		.gp = IMX_GPIO_NR(1, 31)
	},
};

#define EEPROM_I2C_SLAVE_ADDR	0x50

#define EEPROM_ADDR_VENDOR	0x00
#define EEPROM_ADDR_PRODUCT	0x02
#define EEPROM_ADDR_REVISION	0x04
#define EEPROM_ADDR_SERIAL	0x06

#define EEPROM_LENGTH_VENDOR	0x02
#define EEPROM_LENGTH_PRODUCT	0x02
#define EEPROM_LENGTH_REVISION	0x02
#define EEPROM_LENGTH_SERIAL	0x06

#define VENDOR_ATMARK_TECHNO			0x0001

#define PRODUCT_ATMARK_TECHNO_A6_SUBBOARD	0x0001

extern int soft_i2c_gpio_sda;
extern int soft_i2c_gpio_scl;

static int a6_subboard_eeprom_read(u8 addr, void *data, u16 size)
{
	u8 *pdata = (u8 *)data;

	while(size--) {
		*pdata = i2c_reg_read(EEPROM_I2C_SLAVE_ADDR, addr);
		addr++;
		pdata++;
	}
	return 0;
}

static void a6_subboard_get_subboard_info(void)
{
	int ret;
	unsigned short vendor_id;
	unsigned short vendor_id_eeprom;
	unsigned short product_id;
	unsigned short product_id_eeprom;
	unsigned short revision;
	unsigned short revision_eeprom;
	unsigned char  serial[EEPROM_LENGTH_SERIAL];
	char revision_str[EEPROM_LENGTH_REVISION*2+1];
	char serial_str[EEPROM_LENGTH_SERIAL*2+1];

	ret = i2c_probe(EEPROM_I2C_SLAVE_ADDR);
	if (ret) {
		/* no eeprom on subboard */
		return;
	}

	ret = a6_subboard_eeprom_read(EEPROM_ADDR_VENDOR, &vendor_id_eeprom,
				      EEPROM_LENGTH_VENDOR);
	if (ret) {
		printf("a6 subboard: failed to read vendor id.\n");
		return;
	}

	vendor_id = be16_to_cpu(vendor_id_eeprom);

	ret = a6_subboard_eeprom_read(EEPROM_ADDR_PRODUCT, &product_id_eeprom,
				      EEPROM_LENGTH_PRODUCT);
	if (ret) {
		printf("a6 subboard: failed to read product id.\n");
		return;
	}

	product_id = be16_to_cpu(product_id_eeprom);

	ret = a6_subboard_eeprom_read(EEPROM_ADDR_REVISION, &revision_eeprom,
				      EEPROM_LENGTH_REVISION);
	if (ret) {
		printf("a6 subboard: failed to read revision.\n");
		return;
	}

	revision = be16_to_cpu(revision_eeprom);

	ret = a6_subboard_eeprom_read(EEPROM_ADDR_SERIAL, serial,
				      EEPROM_LENGTH_SERIAL);
	if (ret) {
		printf("a6 subboard: failed to read S/N.\n");
		return;
	}


	snprintf(revision_str, sizeof(revision_str), "%04X", revision);
	env_set("revision#", revision_str);

	snprintf(serial_str, sizeof(serial_str), "%02X%02X%02X%02X%02X%02X",
		 serial[0], serial[1], serial[2], serial[3], serial[4],
		 serial[5]);
	env_set("serial#", serial_str);

	if (vendor_id == VENDOR_ATMARK_TECHNO &&
	    product_id == PRODUCT_ATMARK_TECHNO_A6_SUBBOARD)
		printf("Armadillo-IoT A6 Subboard\n");
	else {
		printf("VendorID: %04X\n", vendor_id);
		printf("ProductID: %04X\n", product_id);
	}

	printf("Revision: %s\n", revision_str);
	printf("S/N: %s\n", serial_str);
}

static void a6_subboard_i2c_late_init(void)
{
	soft_i2c_gpio_sda = i2c_gpio_pad.sda.gp;
	soft_i2c_gpio_scl = i2c_gpio_pad.scl.gp;

	gpio_request(soft_i2c_gpio_scl, "soft_i2c");
	gpio_request(soft_i2c_gpio_sda, "soft_i2c");
	i2c_force_idle_bus(&i2c_gpio_pad);
}

void a6_subboard_late_init(void)
{
	int ret;
	int i;

	a6_subboard_i2c_late_init();

	for (i = 0; i < CONFIG_SYS_NUM_I2C_BUSES; i++) {
		if (!strcmp(I2C_ADAP_NR(i)->name, "soft00")) {
			ret = i2c_set_bus_num(i);
			if (ret) {
				printf("a6 subboard: failed to i2c_set_bus_num().\n");
				return;
			}

			a6_subboard_get_subboard_info();
			return;
		}
	}
}
